"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const createCacheTimestampComparator_1 = __importDefault(require("./createCacheTimestampComparator"));
const DiskCache_1 = __importDefault(require("./DiskCache"));
const MemoryCache_1 = __importDefault(require("./MemoryCache"));
class LayeredCache {
    constructor() {
        this.layers = [new MemoryCache_1.default(), new DiskCache_1.default()];
    }
    async get(locale) {
        const cache = this.getLastModifiedLayer(locale);
        if (!cache) {
            return Promise.resolve(undefined);
        }
        const [memoryCache, diskCache] = this.layers;
        const value = await cache.get(locale);
        if (cache === diskCache && typeof value !== 'undefined') {
            await memoryCache.set(locale, value);
        }
        return value;
    }
    getLastModifiedTimestamp(locale) {
        const cache = this.getLastModifiedLayer(locale);
        if (!cache) {
            return undefined;
        }
        return cache.getLastModifiedTimestamp(locale);
    }
    has(locale) {
        return this.layers.some((cache) => cache.has(locale));
    }
    isStale(locale) {
        if (this.layers.some((cache) => cache.isStale(locale))) {
            return true;
        }
        if (this.layers.every((cache) => typeof cache.isStale(locale) === 'undefined')) {
            return undefined;
        }
        return false;
    }
    async set(locale, trie) {
        const [memoryCache, diskCache] = this.layers;
        await diskCache.set(locale, trie);
        await memoryCache.set(locale, trie);
    }
    getLastModifiedLayer(locale) {
        const layers = this.layers.filter((cache) => cache.has(locale));
        const [cached] = [...layers].sort((0, createCacheTimestampComparator_1.default)(locale));
        return cached;
    }
}
exports.default = LayeredCache;
