"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("@scrabble-solver/constants");
const CharacterBonus_1 = __importDefault(require("./CharacterBonus"));
const WordBonus_1 = __importDefault(require("./WordBonus"));
class Config {
    static fromJson(json) {
        return new Config(json);
    }
    constructor(config) {
        this.bonuses = getBonuses(config);
        this.config = config;
        this.pointsMap = getPointsMap(this.config);
    }
    get alphabet() {
        return getAlphabet(this.config);
    }
    get bingoScore() {
        return this.config.bingoScore;
    }
    get blankScore() {
        return this.config.blankScore;
    }
    get blanksCount() {
        return this.config.blanksCount;
    }
    get boardSize() {
        return this.config.boardSize;
    }
    get game() {
        return this.config.game;
    }
    get locale() {
        return this.config.locale;
    }
    get twoCharacterTiles() {
        return this.config.tiles.filter((tile) => tile.character.length === 2).map((tile) => tile.character);
    }
    getCellBonus(cell) {
        return this.bonuses.find((bonus) => bonus.matchesCellCoordinates(cell));
    }
    getCellBonusValue(cell) {
        return this.getCellBonus(cell)?.value || constants_1.NO_BONUS;
    }
    getCharacterPoints(character) {
        if (character === null) {
            return undefined;
        }
        if (character === constants_1.BLANK) {
            return this.blankScore;
        }
        return this.pointsMap[character];
    }
    getTwoCharacterTileByPrefix(character) {
        if (character.length !== 1) {
            return undefined;
        }
        return this.twoCharacterTiles.find((characters) => characters.startsWith(character));
    }
    getTilePoints(tile) {
        if (tile === null) {
            return undefined;
        }
        return tile.isBlank ? this.blankScore : this.getCharacterPoints(tile.character);
    }
    hasCharacter(character) {
        return character in this.pointsMap;
    }
    isTwoCharacterTilePrefix(character) {
        return typeof this.getTwoCharacterTileByPrefix(character) !== 'undefined';
    }
    get rackSize() {
        return this.config.rackSize;
    }
    get tiles() {
        return this.config.tiles;
    }
    toJson() {
        return this.config;
    }
}
const getBonuses = (config) => {
    return config.bonuses.map((bonus) => {
        if (bonus.type === constants_1.BONUS_CHARACTER) {
            return new CharacterBonus_1.default(bonus);
        }
        if (bonus.type === constants_1.BONUS_WORD) {
            return new WordBonus_1.default(bonus);
        }
        throw new Error(`Unsupported Bonus type: "${bonus.type}"`);
    });
};
const getAlphabet = (config) => config.tiles.map(({ character }) => character);
const getPointsMap = (config) => config.tiles.reduce((pointsMap, { character, score }) => ({
    ...pointsMap,
    [character]: score,
}), {});
exports.default = Config;
