"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Pattern_1 = __importDefault(require("./Pattern"));
class VerticalPattern extends Pattern_1.default {
    clone() {
        return new VerticalPattern(this.board, this.cells.map((cell) => cell.clone()));
    }
    getCollisions() {
        const collisions = [];
        this.cells
            .filter((cell) => cell.isEmpty && (this.board.collidesLeft(cell) || this.board.collidesRight(cell)))
            .forEach((cell) => {
            const row = this.board.getRow(cell.y);
            let x = cell.x - 1;
            while (x >= 0 && row[x].hasTile()) {
                --x;
            }
            const previousCells = row.slice(x + 1, cell.x);
            x = cell.x + 1;
            while (x < row.length && row[x].hasTile()) {
                ++x;
            }
            const nextCells = row.slice(cell.x + 1, x);
            const cells = [...previousCells, cell, ...nextCells];
            if (cells.length > 1) {
                const pattern = new Pattern_1.default(this.board, cells);
                collisions.push(pattern);
            }
        });
        return collisions;
    }
}
exports.default = VerticalPattern;
